@@ -59,6 +59,11 @@ function BrowserGlue() {
                                    getService(Ci.nsIObserverService);
   });
 
+  this.__defineGetter__("_distributionCustomizer", function() {
+    delete this._distributionCustomizer;
+    return this._distributionCustomizer = new DistributionCustomizer()
+  });
+
   this._init();
 }
 
@@ -121,6 +126,10 @@ BrowserGlue.prototype = {
         this._observerService.removeObserver(this, "places-init-complete");
         // no longer needed, since history was initialized completely.
         this._observerService.removeObserver(this, "places-database-locked");
+
+        // Now apply distribution customized bookmarks.
+        // This should always run after Places initialization.
+        this._distributionCustomizer.applyBookmarks();
         break;
       case "places-database-locked":
         this._isPlacesDatabaseLocked = true;
@@ -134,6 +143,12 @@ BrowserGlue.prototype = {
           this._archiveBookmarks();
         }
         break;
+      case "distribution-customization-complete":
+        this._observerService
+            .removeObserver(this, "distribution-customization-complete");
+        // Customization has finished, we don't need the customizer anymore.
+        delete this._distributionCustomizer;
+        break;
     }
   }, 
 
@@ -152,6 +167,7 @@ BrowserGlue.prototype = {
     osvr.addObserver(this, "session-save", false);
     osvr.addObserver(this, "places-init-complete", false);
     osvr.addObserver(this, "places-database-locked", false);
+    osvr.addObserver(this, "distribution-customization-complete", false);
   },
 
   // cleanup (called on application shutdown)
@@ -173,8 +189,7 @@ BrowserGlue.prototype = {
   {
     // apply distribution customizations (prefs)
     // other customizations are applied in _onProfileStartup()
-    var distro = new DistributionCustomizer();
-    distro.applyPrefDefaults();
+    this._distributionCustomizer.applyPrefDefaults();
   },
 
   // profile startup handler (contains profile initialization routines)
@@ -193,8 +208,7 @@ BrowserGlue.prototype = {
 
     // apply distribution customizations
     // prefs are applied in _onAppDefaults()
-    var distro = new DistributionCustomizer();
-    distro.applyCustomizations();
+    this._distributionCustomizer.applyCustomizations();
 
     // handle any UI migration
     this._migrateUI();
@@ -383,10 +397,10 @@ BrowserGlue.prototype = {
       return !this._prefs.getBoolPref("browser.EULA.override");
     } catch (e) { }
 
-//@line 427 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
+//@line 441 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
     // Non-official builds shouldn't shouldn't show the notification.
     return false;
-//@line 430 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
+//@line 444 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
 
     // Look to see if the user has seen the current version or not.
     var currentVersion = this._prefs.getIntPref("browser.rights.version");
@@ -911,7 +925,7 @@ BrowserGlue.prototype = {
     }
   },
 
-//@line 957 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
+//@line 971 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
 
   // this returns the most recent non-popup browser window
   getMostRecentBrowserWindow : function ()
@@ -919,7 +933,7 @@ BrowserGlue.prototype = {
     var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
              getService(Components.interfaces.nsIWindowMediator);
 
-//@line 965 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
+//@line 979 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
     var win = wm.getMostRecentWindow("navigator:browser", true);
 
     // if we're lucky, this isn't a popup, and we can just return this
@@ -933,7 +947,7 @@ BrowserGlue.prototype = {
           win = nextWin;
       }
     }
-//@line 991 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
+//@line 1005 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/nsBrowserGlue.js"
 
     return win;
   },
